import { IReactionDisposer } from 'mobx';
import { Constructor, TypeKeys } from 'web-utility';
import { BaseModel } from '../Base';
import { Filter, ListModel, PageData } from '../List';
import { DataObject, RESTClient } from './type';
export declare function toggle<T extends BaseModel>(property: TypeKeys<T, boolean | number>): (origin: (...data: any[]) => Promise<any>, {}: ClassMethodDecoratorContext) => (this: T, ...data: any[]) => Promise<any>;
export declare function toggleNotification<T extends BaseModel>(message: string | ((instance: T) => string)): (origin: (...data: any[]) => Promise<any>, {}: ClassMemberDecoratorContext) => (this: T, ...data: any[]) => Promise<any>;
export interface PersistMeta<V = any, S = any> {
    key: string;
    value?: V | S;
    set?: (value: V) => S | Promise<S>;
    get?: (value?: S) => V | Promise<V>;
    expireIn?: number;
    disposer?: IReactionDisposer;
}
/**
 * @private
 */
export interface PersistBox<V = any, S = any> {
    value: V | S;
    expireAt?: number;
}
export declare class PersistNode<V = any, S = any> implements PersistMeta<V, S> {
    key: string;
    value?: V | S;
    set?: (value: V) => S | Promise<S>;
    get?: (value?: S) => V | Promise<V>;
    expireIn?: number;
    disposer?: IReactionDisposer;
    constructor(meta: PersistMeta<V, S>);
    save(storeKey: string, value?: V): Promise<void>;
    load(storeKey: string): Promise<void | V | S>;
    destroy(storeKey: string): Promise<void>;
}
export type PersistFieldMeta<V = any, S = any> = Pick<PersistMeta<V, S>, 'set' | 'get' | 'expireIn'>;
export declare function persist<T, V, S>({ expireIn, ...meta }?: PersistFieldMeta<V, S>): ({}: ClassAccessorDecoratorTarget<T, V>, { name, addInitializer }: ClassAccessorDecoratorContext<T, V>) => void;
export declare function restore<T extends object>(classInstance: T, storeKey: string): Promise<void>;
export declare function destroy<T extends object>(classInstance: T, storeKey: string): Promise<void>;
export interface PersistModel {
    restored?: Promise<void>;
}
export interface PersistListMeta<T> extends Pick<PersistMeta, 'expireIn'> {
    storeKey: string | ((instance: T) => string);
}
export declare function persistList<D extends DataObject, F extends Filter<D> = Filter<D>, T extends Constructor<ListModel<D, F>> = Constructor<ListModel<D, F>>>({ storeKey, expireIn }?: PersistListMeta<InstanceType<T>>): (Super: T, {}: ClassDecoratorContext) => {
    new (...args: any[]): {
        client: RESTClient;
        baseURI: string;
        pageIndex: number;
        pageSize: number;
        filter: F;
        totalCount: number | undefined;
        pageList: D[][];
        restored: Promise<void>;
        loadPage: (pageIndex: number, pageSize: number, filter: F) => Promise<PageData<D>>;
        readonly currentPage: D_1[];
        readonly pageCount: number;
        readonly allItems: D_1[];
        readonly noMore: boolean;
        statistic: Partial<Record<TypeKeys<D, import("web-utility").IndexKey>, Record<string, number>>>;
        clearList(): void;
        clear(): void;
        restoreList({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D, Partial<import("./type").NewData<D>>>, "pageIndex" | "pageSize" | "totalCount" | "allItems">>): void;
        turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
        loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
            pageData: D[];
            totalCount: number;
        }>;
        getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
        refreshList(): Promise<D[]>;
        getAll(filter?: F, pageSize?: number): Promise<D[]>;
        countAll(keys: TypeKeys<D, import("web-utility").IndexKey>[], filter?: F, pageSize?: number): Promise<Partial<Record<TypeKeys<D, import("web-utility").IndexKey>, Record<string, number>>>>;
        indexOf(id: import("./type").IDType): number;
        changeOne(data: Partial<D>, id: import("./type").IDType, patch?: boolean): void;
        updateOne(data: Partial<import("./type").NewData<D>>, id?: import("./type").IDType): Promise<D>;
        removeOne(id: import("./type").IDType): Promise<void>;
        deleteOne(id: import("./type").IDType): Promise<void>;
        indexKey: keyof D;
        currentOne: D_1;
        clearCurrent(): void;
        getOne(id: import("./type").IDType): Promise<D>;
        downloading: number;
        uploading: number;
    };
} & T;
